/********************************************************************/
/* Licensed Materials - Property of IBM                             */
/* IBM MASS Library Component V8.1.3 for Linux (5725-C75, 5765-J10) */
/* Copyright IBM Corp. 2007, 2013.                                  */
/* US Government Users Restricted Rights - Use, duplication or      */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.*/
/********************************************************************/

#pragma options float=rsqrt libansi
#define __TWO52 4503599627370496.0
#include <math.h>

#ifdef __nocomplex

  void vcosisin (double y[][2], double x[], int *n)
  {
    int i;
    for (i=0; i<*n; i++){
      y[i][0] = cos(x[i]);
      y[i][1] = sin(x[i]);
    }
  }
  
  void vscosisin(float y[][2], float x[], int *n)
  {
    int i;
    for (i=0; i<*n; i++){
      y[i][0] = cos(x[i]);
      y[i][1] = sin(x[i]);
    }
  }

#else

  #ifdef __cplusplus
  #else
    #include <complex.h>
  #endif

  void vcosisin (double _Complex y[], double x[], int *n)
    /* Sets the real part of y[i] to the cosine of x[i] and the imaginary 
       part of y[i] to the sine of x[i], for i=0,..,n-1 */
  {
    int i;
    for (i=0; i<*n; i++){
     y[i]=cos(x[i])+sin(x[i])*_Complex_I;
    }
  }
  
  void vscosisin (float _Complex y[], float x[], int *n)
    /* Sets the real part of y[i] to the cosine of x[i] and the imaginary 
       part of y[i] to the sine of x[i], for i=0,..,n-1 */
  {
    int i;
    for (i=0; i<*n; i++){
      y[i]=cos(x[i])+sin(x[i])*_Complex_I;
    }
  }

#endif

void vrec (double y[], double x[], int *n) 
{
  /* Sets y[i] to the reciprocal of x[i], for i=0,..,n-1 */
  int i;
  for (i=0; i<*n; i++)
    y[i] = 1.0/x[i];
}
  
void vsqrt (double y[], double x[], int *n) 
{
  /* Sets y[i] to the square root of x[i], for i=0,..,n-1 */
  int i;
  for (i=0; i<*n; i++)
    y[i] = sqrt(x[i]);
}

void vrsqrt (double y[], double x[], int *n) 
{
  /* Sets y[i] to the reciprocal of the square root of x[i], for i=0,..,n-1 */
  int i;
  for (i=0; i<*n; i++)
    y[i] = 1.0/sqrt(x[i]);
}

void vsrec (float y[], float x[], int *n) 
{
  /* Sets y[i] to the reciprocal of x[i], for i=0,..,n-1 */
  int i;
  for (i=0; i<*n; i++)
    y[i] = 1.0f/x[i];
}

void vssqrt (float y[], float x[], int *n) 
  /* Sets y[i] to the square root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = sqrt(x[i]);
}

void vsrsqrt (float y[], float x[], int *n) 
{
  /* Sets y[i] to the reciprocal of the square root of x[i], for i=0,..,n-1 */
  int i;
  for (i=0; i<*n; i++)
    y[i] = 1.0f/sqrt(x[i]);
}


void vcbrt (double y[], double x[], int *n)
  /* Sets y[i] to the cube root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], 1.0/3.0);
}
void vrcbrt (double y[], double x[], int *n)
  /* Sets y[i] to the reciprocal of the cube root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], -1.0/3.0);
}
void vscbrt (float y[], float x[], int *n)
  /* Sets y[i] to the cube root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], 1.0/3.0);
}
void vsrcbrt (float y[], float x[], int *n)
  /* Sets y[i] to the reciprocal of the cube root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], -1.0/3.0);
}

void vqdrt (double y[], double x[], int *n)
  /* Sets y[i] to the quad root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], 1.0/4.0);
}
void vrqdrt (double y[], double x[], int *n)
  /* Sets y[i] to the reciprocal of the quad root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], -1.0/4.0);
}
void vsqdrt (float y[], float x[], int *n)
  /* Sets y[i] to the quad root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], 1.0/4.0);
}
void vsrqdrt (float y[], float x[], int *n)
  /* Sets y[i] to the reciprocal of the quad root of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = pow(x[i], -1.0/4.0);
}

void vacosh (double y[], double x[], int *n)
  /* Sets y[i] to the hyperbolic arccosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = acosh(x[i]);
}

void vasinh (double y[], double x[], int *n)
  /* Sets y[i] to the hyperbolic arcsine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = asinh(x[i]);
}

void vatanh (double y[], double x[], int *n)
  /* Sets y[i] to the hyperbolic arctan of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = atanh(x[i]);
}

void vsacosh (float y[], float x[], int *n)
  /* Sets y[i] to the hyperbolic arccosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = acosh(x[i]);
}

void vsasinh (float y[], float x[], int *n)
  /* Sets y[i] to the hyperbolic arcsine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = asinh(x[i]);
}

void vsatanh (float y[], float x[], int *n)
  /* Sets y[i] to the hyperbolic arctan of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = atanh(x[i]);
}


void vacos (double y[], double x[], int *n)
  /* Sets y[i] to the arccosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = acos(x[i]);
}

void vasin (double y[], double x[], int *n)
  /* Sets y[i] to the arcsine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = asin(x[i]);
}

void vatan2 (double z[], double x[], double y[], int *n)
  /* Sets y[i] to the arctangent of x[i]/y[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    z[i] = atan2(x[i],y[i]);
}

void vcos (double y[], double x[], int *n)
  /* Sets y[i] to the cosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = cos(x[i]);
}

void vcosh (double y[], double x[], int *n)
  /* Sets y[i] to the hyperbolic cosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = cosh(x[i]);
}


void vdiv (double z[], double x[], double y[], int *n) 
  /* Sets z[i] to x[i]/y[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    z[i] = x[i]/y[i];
}


void vexp (double y[], double x[], int *n) 
  /* Sets y[i] to the exponential function of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = exp(x[i]);
}

void vexpm1 (double y[], double x[], int *n) 
  /* Sets y[i] to (the exponential function of x[i])-1, for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = expm1(x[i]);
}


void vlog (double y[], double x[], int *n) 
  /* Sets y[i] to the natural logarithm of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = log(x[i]);
}


void vlog10 (double y[], double x[], int *n) 
  /* Sets y[i] to the base-10 logarithm of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = log10(x[i]);
}

void vlog1p (double y[], double x[], int *n) 
  /* Sets y[i] to the natural logarithm of (x[i]+1), for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = log1p(x[i]);
}

void vpow (double z[], double x[], double y[], int *n)
  /* Sets z[i] to x[i] raised to the power y[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    z[i] = pow(x[i],y[i]);
}

void vsin (double y[], double x[], int *n)
  /* Sets y[i] to the sine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = sin(x[i]);
}


void vsincos (double y[], double z[], double x[], int *n)
  /* Sets y[i] to the sine of x[i] and z[i] to the cosine of x[i], 
     for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++){
    y[i] = sin(x[i]);
    z[i] = cos(x[i]);
  }
}

void vsinh (double y[], double x[], int *n)
  /* Sets y[i] to the hyperbolic sine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = sinh(x[i]);
}

void vtan (double y[], double x[], int *n)
  /* Sets y[i] to the tangent of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = tan(x[i]);
}

void vtanh (double y[], double x[], int *n)
  /* Sets y[i] to the hyperbolic tangent of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = tanh(x[i]);
}

void vsacos (float y[], float x[], int *n)
  /* Sets y[i] to the arccosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = acos(x[i]);
}

void vsasin (float y[], float x[], int *n)
  /* Sets y[i] to the arcsine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = asin(x[i]);
}

void vsatan2 (float z[], float x[], float y[], int *n)
  /* Sets z[i] to the arctangent of x[i]/y[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    z[i] = atan2(x[i],y[i]);
}

void vscos (float y[], float x[], int *n)
  /* Sets y[i] to the cosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = cos(x[i]);
}

void vscosh (float y[], float x[], int *n)
  /* Sets y[i] to the hyperbolic cosine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = cosh(x[i]);
}


void vsdiv (float z[], float x[], float y[], int *n) 
  /* Sets z[i] to x[i]/y[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    z[i] = x[i]/y[i];
}

void vsexp (float y[], float x[], int *n) 
  /* Sets y[i] to the exponential function of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = exp(x[i]);
}

void vsexpm1 (float y[], float x[], int *n) 
  /* Sets y[i] to (the exponential function of x[i])-1, for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = expm1(x[i]);
}

void vslog (float y[], float x[], int *n) 
  /* Sets y[i] to the natural logarithm of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = log(x[i]);
}

void vslog10 (float y[], float x[], int *n) 
  /* Sets y[i] to the base-10 logarithm of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = log10(x[i]);
}

void vslog1p (float y[], float x[], int *n) 
  /* Sets y[i] to the natural logarithm of (x[i]+1), for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = log1p(x[i]);
}

void vspow (float z[], float x[], float y[], int *n)
  /* Sets z[i] to x[i] raised to the power y[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    z[i] = pow(x[i],y[i]);
}

void vssin (float y[], float x[], int *n)
  /* Sets y[i] to the sine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = sin(x[i]);
}

void vssincos (float y[], float z[], float x[], int *n)
  /* Sets y[i] to the sine of x[i] and z[i] to the cosine of x[i], 
     for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++){
    y[i] = sin(x[i]);
    z[i] = cos(x[i]);
  }
}

void vssinh (float y[], float x[], int *n)
  /* Sets y[i] to the hyperbolic sine of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = sinh(x[i]);
}


void vstan (float y[], float x[], int *n)
  /* Sets y[i] to the tangent of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = tan(x[i]);
}

void vstanh (float y[], float x[], int *n)
  /* Sets y[i] to the hyperbolic tangent of x[i], for i=0,..,n-1 */
{
  int i;
  for (i=0; i<*n; i++)
    y[i] = tanh(x[i]);
}

double dnint (double *x)
{
  double absx, y;
  long long ll;

  absx = fabs(*x);
  if (absx >= __TWO52)
  {
    /* x is an integer */
    return *x;
  }

  /* Otherwise, x fits into a 64-bit integer */
  ll = absx + 0.5;

  if (*x > 0)
    y = ll;
  else
    y = -ll;

  return y;
}


void vdnint (double y[], double x[], int* n)
{
  int i;
  double dnint (double *x);

  for (i=0; i<*n; i++)
    y[i] = dnint (&(x[i]));
}

void vdint (double y[], double x[], int* n)
{
  int i;

  double __dint (double x)
  {
    double absx, y;
    long long ll;

    absx = fabs(x);
    if (absx >= __TWO52)
    {
      /* x is an integer */
      return x;
    }

    /* Otherwise, x fits into a 64-bit integer */
    ll = absx;

    if (x > 0)
      y = ll;
    else
      y = -ll;

    return y;
  }

  for (i=0; i<*n; i++)
    y[i] = __dint (x[i]);
}
